# -*- coding: utf-8 -*-
# Author: MangoGO (https://blog.csdn.net/silencewx1005)
# Time  : 2024/3/1 21:50
# Desc  :
import time

import torch
from torch.utils.data import DataLoader
from transformers import BertTokenizer

from data import DFData, CommentDataset, comment_collate_func
from model import CustomConfig, CommentModel
from parameters import Parameter


if __name__ == '__main__':
    parameter = Parameter()
    paras = parameter.args
    start_min = time.strftime("%Y%m%d%H%M", time.localtime())

    dev = torch.device(paras.get('device') if torch.cuda.is_available()
                       else 'cpu')

    model_config = CustomConfig.from_pretrained_with_define(
        paras.get('infer_model_path'), paras)
    tokenizer = BertTokenizer.from_pretrained(paras.get('infer_model_path'))

    infer_df = DFData(paras.get('train_data_path')).get_data()

    infer_dataset = CommentDataset(infer_df, tokenizer, paras.get('max_len'))
    infer_dataloader = DataLoader(infer_dataset,
                                  batch_size=30,
                                  collate_fn=comment_collate_func)

    comment_model = CommentModel(paras.get('infer_model_path'), model_config,
                                 paras.get('model_out_size')).to(dev)
    comment_model.load_model(paras.get('infer_model_state_path'), dev)

    # cust_sentence_df = pd.DataFrame({'comment': [
    #     '从头到尾不知道在说什么',
    #     '神作啊，全程高能',
    #     '一般般',
    #     '太好看了'
    # ]})
    # cust_dataset = CommentDataset(cust_sentence_df, tokenizer,
    #                               paras.get('max_len'))
    # cust_dataloader = DataLoader(cust_dataset,
    #                              batch_size=cust_sentence_df.shape[0],
    #                              collate_fn=comment_collate_func)

    pre_res = []
    comment_model.eval()
    with torch.no_grad():
        for batch in infer_dataloader:
            (ids, mask, type_ids, label) = batch
            ids, mask, type_ids = ids.to(dev), mask.to(dev), type_ids.to(dev)

            tmp_res = comment_model(ids, mask, type_ids)
            tmp_res = torch.max(tmp_res.cpu(), 1)[1].numpy().tolist()
            pre_res.extend(tmp_res)

    infer_df['infer_res'] = pre_res

    infer_df['is_same'] = (infer_df['label'] - infer_df['infer_res'] == 0)
    infer_df.to_excel(paras.get('infer_res_path'), index=False)
