# -*- coding: utf-8 -*-
# Author: MangoGO (https://blog.csdn.net/silencewx1005)
# Time  : 2024/1/24 22:00
# Desc  : Reading system and custom parameters

import argparse
import configparser

from utils import dict2type


class Parameter:
    def __init__(self, config_path='./config.ini'):
        self.parser = argparse.ArgumentParser()
        self.arg_system()
        args, unknown = self.parser.parse_known_args()
        self.args = args.__dict__
        self.config_path = config_path

        self.paras_val, self.paras_type = {}, {}
        self.update_args()
        self.logic_paras()

    def arg_system(self):
        self.parser.add_argument('--EXTRA_PARAS', type=str,
                                 help='formatted like '
                                      '"[sep]--key:value [sep]value type"')
        self.parser.add_argument('--OUTPUT_PATH', type=str,
                                 help='output file path')

    def extra_parser(self):
        extra_strs = self.args.get('extra_paras')
        if extra_strs is None:
            return
        extra_strs = [item.strip() for item in extra_strs.split('--')
                      if item.strip() != '' and item.strip()[0] != ':']
        if len(extra_strs) < 2:
            raise ValueError('Parameter extra_paras got wrong value！')

        sep, k_v_ts = extra_strs[0], extra_strs[1:]
        dicts = {}
        for k_v_t in k_v_ts:
            k_vt = [item.strip() for item in k_v_t.split(':')
                    if item.strip() != ''][:2]
            dicts[k_vt[0]] = k_vt[1] if len(k_vt) == 2 else ''
        configs, dtypes = self.str2kv(dict(dicts), sep)

        self.convert_type(configs, dtypes)

    def get_config(self):
        conf = configparser.ConfigParser()
        conf.read(self.config_path, encoding="utf-8")

        self.args['config_sep'] = conf['DEFAULT']['config_sep']

        for section, dicts in conf.items():
            if dicts is None or section == 'DEFAULT':
                continue

            configs, dtypes = self.str2kv(dict(dicts), self.args['config_sep'])

            self.convert_type(configs, dtypes)

    @staticmethod
    def str2kv(key_val_typ, sep):
        configs, dtypes = {}, {}
        for para, val_typ in key_val_typ.items():
            val_typ = [i.strip() for i in val_typ.split(sep)][:2]
            configs[para] = val_typ[0]
            dtypes[para] = 'str' if len(val_typ) == 1 else val_typ[1]

        return configs, dtypes

    def convert_type(self, configs, dtypes):
        dtypes = dict2type(dtypes)
        for k, v in configs.items():
            if v.lower() in ('', 'none'):
                self.paras_val[k] = None
            else:
                configs[k] = dtypes[k](v)

        self.paras_val = dict(configs, **self.paras_val)
        self.paras_type = dict(dtypes, **self.paras_type)

    def update_args(self):
        self.extra_parser()
        self.get_config()
        self.args = dict(self.paras_val, **self.args)

    def logic_paras(self):
        self._device()

    def show_args(self):
        print('Get system and custom parameters:')
        for k, v in self.args.items():
            print(f'    --{k}: {v}, type: {type(v)}')

    def _device(self):
        device = self.args.get('device')
        if device is not None and device == 'gpu':
            self.args['device'] = 'cuda'


if __name__ == '__main__':
    parameter = Parameter()
    parameter.show_args()

